<?php

namespace Teknisa\CRM\Service;

use Teknisa\Model\Util\NovoCodigo;

class crm00002001OportunidadeNegocioService
{

    protected $oportFactory;
    protected $environment;

    public function __construct($oportFactory, $environment)
    {

        $this->oportFactory = $oportFactory;
        $this->environment  = $environment;
    }

    public function isOperadorMaster($rowIsMaster)
    {
        $operadorIsMaster = $rowIsMaster[0] == 'S';

        return $operadorIsMaster;
    }

    public function transferirOportunidade($row)
    {
        $operador = $row["OPERADOR"];
        $row = $row["ROW"];
        $paramsTransferirOportu = array(
            "NRPARCRESP"    => $operador,
            "NRSEQOPORT"    => $row["NRSEQOPORT"],
            "NRORG"         => $row["NRORG"],
            "CRM_NRORGTRAB" => $row["CRM_NRORGTRAB"]
        );
        $this->oportFactory->transferirOportunidade($paramsTransferirOportu);
    }

    public function evolTransferirOportunidade($row)
    {
        $operador = $row["OPERADOR"];
        $nmoperador = $row["NOVORESPONSAVEL"];
        $row = $row["ROW"];

        $textEvol = "Oportunidade transferida de responsável. (De " . $row["NMPARCRESP"] . " para " . $nmoperador . ")";
        $row["NRSEQEVOLCONT"] = NovoCodigo::geraCodigo($row["NRORG"], "EVOLCONT", 1, 10, false, false);

        $paramsEvolTransferirOportu = array(
            "NRCONTATO"         => $row["NRCONTATO"],
            "NRSEQCONTATO"      => $row["NRCONTATO"],
            "IDSENEVOCONT"      => "S",
            "IDTPEVOLCONT"      => "P",
            "IDAGEEVOCONT"      => "N",
            "CDOPEREVOL"        => $row["CDOPERADOR"],
            "IDANEXOEVOLCONT"   => "N",
            "IDGERAFATURA"      => "NAO",
            "IDEVOLPUBLICO"     => "S",
            "IDRESPNOTA"        => "R",
            "IDEVOLINTERNA"     => "S",
            "DSEVOLCONT"        => $textEvol,
            "TXEVOLCONT"        => $textEvol,
            "NRSEQEVOLCONT"     => $row["NRSEQEVOLCONT"],
            "IDATIVO"           => "S",
            "CDOPERINCLUSAO"    => $row["CDOPERADOR"],
            "NRORG"             => $row["NRORG"],
            "CRM_NRORGTRAB"     => $row["CRM_NRORGTRAB"]
        );

        $this->oportFactory->evolTransferirOportunidade($paramsEvolTransferirOportu);
    }

    public function setAuditColumns($row)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorgOper  = $this->environment->getNrorg();

        $row['DTULTATU']     = (new \DateTime)->format('d/m/Y H:i:s'); //SYSDATE
        $row['CDOPERULTATU'] = $cdoperador;
        $row['NRORGULTATU']  = $nrorgOper;
    }

    public function buscarOportunidade($row)
    {
        $params = array(
            "P_NRSEQOPORT"      => $row['NRSEQOPORT'],
            "P_NRORG"           => $row['NRORG'],
            "P_CRM_NRORGTRAB"   => $row['CRM_NRORGTRAB']
        );

        return $this->oportFactory->buscarOportunidade($params);
    }

    public function closeOportByOccurrenceService($row)
    {

        $params = array(
            "IDATIVO"       => $row['IDATIVO'],
            "DTREALFECHA"   => $row['DTREALFECHA'],
            "NRSEQFASENEGO" => $row['NRSEQFASENEGO'],
            "CDOPERULTATU"  => $row['CDOPERULTATU'],
            "NRSEQOPORT"    => $row['NRSEQOPORT'],
            "NRPARCRESP"    => $row['NRPARCRESP'],
            "CRM_NRORGTRAB" => $row['CRM_NRORGTRAB']
        );

        return $this->oportFactory->closeOportByOccurrenceFactory($params);
    }

    public function setEvolutionService($row)
    {

        $row["NRSEQEVOLCONT"] = NovoCodigo::geraCodigo($row["NRORG"], "EVOLCONT", 1, 12, false, false);

        $params = array(
            "NRCONTATO"         => $row["NRCONTATO"],
            "NRSEQCONTATO"      => $row["NRSEQCONTATO"],
            "IDSENEVOCONT"      => "S",
            "IDTPEVOLCONT"      => "P",
            "IDAGEEVOCONT"      => "N",
            "CDOPEREVOL"        => $row["CDOPERADOR"],
            "CDOPERULTATU"      => $row["CDOPERULTATU"],
            "NRSEQFASENEGO"     => $row["NRSEQFASENEGO"],
            "NRTERMOPORTUNI"    => $row["NRTERMOPORTUNI"],
            "CDTIPOCONT"        => "71",
            "NRSEQRESULEVOL"    => "7",
            "CDMOTIEVOCON"      => "00999",
            "IDANEXOEVOLCONT"   => "N",
            "IDGERAFATURA"      => "NAO",
            "IDEVOLPUBLICO"     => "S",
            "IDRESPNOTA"        => "R",
            "IDEVOLINTERNA"     => "S",
            "DSEVOLCONT"        => "Oportunidade Encerrada Pelo(a) " . $row["DSHIERARQUIAOPERPAI"],
            "TXEVOLCONT"        => "Oportunidade Encerrada Pelo(a) " . $row["DSHIERARQUIAOPERPAI"],
            "NRSEQEVOLCONT"     => $row["NRSEQEVOLCONT"],
            "IDATIVO"           => "N",
            "CDOPERINCLUSAO"    => $row["CDOPERADOR"],
            "NRORG"             => $row["NRORG"],
            "CRM_NRORGTRAB"     => $row["CRM_NRORGTRAB"]
        );

        $this->oportFactory->setEvolutionFactory($params);
        return ["Success!"];
    }

    public function callForConcludeOportTaskService($row, $nrFilaConta)
    {

        $paramsDeactivateTask = array(

            "NRSEQFILACONTA" => $nrFilaConta["NRSEQFILACONTA"],
            "NRORG"          => $row["NRORG"],
            "DSDETEFILACONTA"=> "Tarefa Encerrada Pelo(a) " . $row["DSHIERARQUIAOPERPAI"]

        );
   
        $this->oportFactory->callForConcludeOportTaskFactory($paramsDeactivateTask);

    }

    public function findTasksService($row)
    {

        $params = array(
            "NRSEQCONTATO" => $row["NRSEQCONTATO"] 
        );
      
        try {

            return $this->oportFactory->findTasksFactory($params);

        } catch (\Exception $e) {

            throw $e;
            
        }
    }
}
