function ABALogotipoParceiroController(CRM_Utilities, templateManager, ScreenService, Messages, General, metaDataFactory, restEngine) {
    
    /**
     * Insere a widget de logo na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a widget de logotipo
     * ("WDGT_LOGO_PARCEIRO").
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */
    this.setLogoWidgetOnActiveWidget = function(widget, position){
        
        widget.widgets = CRM_Utilities.setElementOnArrayDefinedPosition(this.factoryLogoWidget(widget), widget.widgets, position);
        widget.activate();
        
    };
    
    
    /**
     * builda a widget de logotipo.
     * 
     * @param {object} widget widget ativa.
     * 
     * @return {object} widget de logotipo construída.
    */
    this.factoryLogoWidget = function(activeWidget){
        
        var logoWdgtJson = {
            "id": "17223197583113153905177",
            "name": "WDGT_LOGO_PARCEIRO",
            "label": "Logotipo",
            "template": "widget/form-view.html",
            "container": activeWidget.container,
            "parent": activeWidget,
            "showCheckbox": false,
            "isVisible": true,
            "dataSource": {
                "name": "product#/parcnegocio_logotipo",
                "rest": true,
                "lazyLoad": true
            },
            "dataSourceFilter": [
                {
                    "name": "NRPARCNEGOCIO",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": "@getLocalVar('CRM_NRORG')@"
                }
            ],
            "events": [
                {
                    "id": "17223197583113153905182",
                    "name": "WidgetAfterLoadDataSource",
                    "code": "ABALogotipoParceiroController.wdgLogotipoAfterLoadDataSource(args.owner);"
                },
                {
                    "id": "17223197583113153905183",
                    "name": "WidgetOnEnterTab",
                    "code": "ABALogotipoParceiroController.setLogotipoWidgetFilter(args.owner);ABALogotipoParceiroController.wdgLogotipoOnEnter(args.owner);"
                }
            ],
            "actions": [
                {
                    "id": "17223197583113153905178",
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back_list|back_view",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closeSwipe();"
                        }
                    ]
                },
                {
                    "id": "17223197583113153905179",
                    "name": "edit",
                    "label": "Editar",
                    "icon": "pencil",
                    "showAsAction": "view",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABALogotipoParceiroController.editarLogo(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "17223197583113153905180",
                    "name": "delete",
                    "label": "Excluir",
                    "icon": "trash",
                    "showAsAction": "view",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABALogotipoParceiroController.excluirLogo(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "17223197583113153905181",
                    "name": "cancel",
                    "label": "Cancelar",
                    "hideIcon": true,
                    "showAsAction": "back_edit",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABALogotipoParceiroController.cancelarLogo(args.owner.widget);"
                        }
                    ]
                },
                {
                    "id": "17223197583113153905182",
                    "name": "save",
                    "label": "Salvar",
                    "showAsAction": "never_edit",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABALogotipoParceiroController.salvarLogo(args.owner.widget);"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": "17223197583113153905184",
                    "name": "NRPARCNEGOCIO",
                    "isVisible": false
                },
                {
                    "id": "17223197583113153905185",
                    "name": "NRORG",
                    "isVisible": false
                },
                {
                    "id": "17223197583113153905186",
                    "name": "DSDIRLOGOMARCA_OLD",
                    "isVisible": false
                },
                {
                    "id": "17223197583113153905187",
                    "name": "DSDIRLOGOMARCA",
                    "label": "Logotipo",
                    "template": "zh-image-uploader#field/image-uploader.html",
                    "class": "12",
                    "preview": "DSDIRLOGOMARCA",
                    "isVisible": true,
                    "validations": {
                        "extensions": {
                            "value": [
                                ".jpg", ".JPG",
                                ".jpeg", ".JPEG",
                                ".png", ".PNG",
                                ".bmp", ".BMP"
                            ]
                        }
                    }
                }
            ],
            "widgets": []
        };
        
        var logoWdgt = metaDataFactory.widgetFactory(logoWdgtJson);
        CRM_Utilities.factoryWidgetActions(logoWdgt);
        
        return logoWdgt;
        
    };
    
    /**
     * Define o filtro da aba de logotipo pelo
     * parceiro de negócio em caso.
     * 
     * @param {object} widget widget de logotipo.
    */
    this.setLogotipoWidgetFilter = function(widget){
        
        var parentWidgetCurrentRow = widget.getParent().currentRow;
        /*Adaptação para widget modal 'dados do parceiro'*/
        if(Util.isEmptyOrBlank(Object.keys(parentWidgetCurrentRow))){
            parentWidgetCurrentRow = widget.getParent().widgets[0].currentRow;
        }
        var nrparcnegocioFilterIndex = CRM_Utilities.getFilterPosition(widget.dataSourceFilter, "NRPARCNEGOCIO");
        if(!Boolean(widget.NRPARCEIROFILTRADO) || widget.NRPARCEIROFILTRADO !== parentWidgetCurrentRow.NRPARCNEGOCIO){
            
             widget.dataSourceFilter[nrparcnegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
             widget.NRPARCEIROFILTRADO = parentWidgetCurrentRow.NRPARCNEGOCIO;
        
             widget.reload();
            
        }
        
    };
    
    this.wdgLogotipoAfterLoadDataSource = function(wdgLogotipo) {
        wdgLogotipo.currentRow = wdgLogotipo.dataSource.data[0];
        this.controlarAcaoExcluirLogo(wdgLogotipo);
        this.buildImageInViewMode(wdgLogotipo);
    };
    
    this.controlarAcaoExcluirLogo = function(wdgLogotipo) {
        var actExcluir = wdgLogotipo.getAction("delete");
        actExcluir.isVisible = (!wdgLogotipo.currentRow.DSDIRLOGOMARCA) ? false : true;
    };
    
    this.wdgLogotipoOnEnter = function(wdgLogotipo) {
        wdgLogotipo.list();
        wdgLogotipo.reload();
        wdgLogotipo.view();
    };
    
    this.editarLogo = function(wdgLogotipo) {
        var nrParcNegocio = wdgLogotipo.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = wdgLogotipo.parent.currentRow.CDTIPOPARCPRINCIPAL;
    
        if (!nrParcNegocio) {
            nrParcNegocio = wdgLogotipo.parent.parent.currentRow.NRPARCNEGOCIO;
        }
    
        if (!tipoParc) {
            if (templateManager.container.name == 'ger45000_prospect') {
                tipoParc = "PROSPECT";
            }
            if (templateManager.container.name == 'crm_cliente') {
                tipoParc = "CLIENTE";
            }
        }
    
        if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
            this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
                if (canEdit) {
                    wdgLogotipo.backupRow = Util.clone(wdgLogotipo.currentRow);
                    wdgLogotipo.edit();
                    wdgLogotipo.template = "widget/form.html";
                } else {
                    this.showMessageCantEditParc();
                }
            }.bind(this));
        } else {
            wdgLogotipo.backupRow = Util.clone(wdgLogotipo.currentRow);
            wdgLogotipo.edit();
            wdgLogotipo.template = "widget/form.html";
        }
    
        var widgets = this.getWidgetsMenosLogo();
        General.changeWidgetAcessibility(widgets, false);
    };
    
    this.cancelarLogo = function(wdgLogotipo) {
        Object.keys(wdgLogotipo.backupRow).forEach(function (key) {
			wdgLogotipo.currentRow[key] = wdgLogotipo.backupRow[key];
		}); 
		wdgLogotipo.view();
		wdgLogotipo.template = "widget/form-view.html";
		
		var widgets = this.getWidgetsMenosLogo();
		General.changeWidgetAcessibility(widgets, true);
		this.buildImageInViewMode(wdgLogotipo);
    };
    
    this.buildImageInViewMode = function(wdgLogotipo){
        setTimeout(function(){
            if(!Util.isEmptyOrBlank(wdgLogotipo.dataSource.data[0]['DSDIRLOGOMARCA'])){
                var row = wdgLogotipo.dataSource.data[0]['DSDIRLOGOMARCA'];
                var fieldLogo = $('#span-field-DSDIRLOGOMARCA');
                var b64Image = typeof row === "object" ? row[0]['b64File'] : row;
                var urlLogo = window.fileServerBaseUrl+b64Image;
                var image = '<img src='+urlLogo+' class="img-responsive center-block">';
                if($('.img-responsive.center-block').length <= 0){
                    fieldLogo.find('img').remove();
                    fieldLogo.append(image);
                    fieldLogo.find('p').css({display:'none'});
                }
            }else{
                this.setDefaultImage();
            }
        }.bind(this), 50);
    };
    
    this.setDefaultImage = function(){
        var fieldLogo = $('#span-field-DSDIRLOGOMARCA');
        fieldLogo.find('p').css({display:'none'});
        fieldLogo.find('img').remove();
        fieldLogo.append('<img></img>');
        fieldLogo.find('img').addClass('defaultImage');
    }
    
    this.salvarLogo = function(wdgLogotipo) {
        var nrParcNegocio = wdgLogotipo.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = wdgLogotipo.parent.currentRow.CDTIPOPARCPRINCIPAL;
        var currentRow = wdgLogotipo.currentRow;
        currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
    
        if (!nrParcNegocio) {
            nrParcNegocio = wdgLogotipo.parent.parent.currentRow.NRPARCNEGOCIO;
        }
    
        if (!tipoParc) {
            if (templateManager.container.name == 'ger45000_prospect') {
                tipoParc = "PROSPECT";
            }
            if (templateManager.container.name == 'crm_cliente') {
                tipoParc = "CLIENTE";
            }
        }
    
        var verifyGroupRelCallback = function(response) {
            if (response.dataset && response.dataset.error) {
                ScreenService.showMessage(response.dataset.error, 'ERROR');
            }
    
            if (response.dataset && response.dataset.GROUPREL[0] == false) {
                var logo = wdgLogotipo.currentRow.DSDIRLOGOMARCA;
    
                if (logo instanceof Array && logo.length > 0) {
                    General.widgetSave(wdgLogotipo).then(
                        function () {
                            this.controlarAcaoExcluirLogo(wdgLogotipo);
    
                            var widgets = this.getWidgetsMenosLogo();
                            General.changeWidgetAcessibility(widgets, true);
    
                            wdgLogotipo.reload().then(
                                function () {
                                    this.wdgLogotipoAfterLoadDataSource(wdgLogotipo);
                                }.bind(this)
                            );
                        }.bind(this),
                        function (error) {
                            console.error(error);
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                } else {
                    ScreenService.showMessage("Escolha um arquivo para salvar.");
                }
            } else {
                ScreenService.showMessage(
                    'Operação Bloqueada! O parceiro de negócio está relacionado ao parceiro:<br>' +
                        response.dataset.GROUPREL[0]['NRINSCRICAOPARC'] +
                        ' - ' +
                        response.dataset.GROUPREL[0]['NMPRINCIPALPARC'] +
                        '!'
                );
            }
        }.bind(this);
    
        if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
            this.checkIfCanEditOrEvolParc(nrParcNegocio).then(
                function (canEdit) {
                    if (canEdit) {
                        General.callRestEngine("product#/verifyGroupRel", "DataSet", [currentRow]).then(
                            verifyGroupRelCallback
                        );
                    } else {
                        this.showMessageCantEditParc();
                    }
                }.bind(this)
            );
        } else {
            General.callRestEngine("product#/verifyGroupRel", "DataSet", [currentRow]).then(
                verifyGroupRelCallback
            );
        }
    };
    
    this.excluirLogo = function(wdgLogotipo) {
        var nrParcNegocio = wdgLogotipo.parent.currentRow.NRPARCNEGOCIO;
        var tipoParc = wdgLogotipo.parent.currentRow.CDTIPOPARCPRINCIPAL;
        var currentRow = wdgLogotipo.currentRow;
        currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
    
        if (!nrParcNegocio) {
            nrParcNegocio = wdgLogotipo.parent.parent.currentRow.NRPARCNEGOCIO;
        }
    
        if (!tipoParc) {
            if (templateManager.container.name == 'ger45000_prospect') {
                tipoParc = "PROSPECT";
            }
            if (templateManager.container.name == 'crm_cliente') {
                tipoParc = "CLIENTE";
            }
        }
    
        var verifyGroupRelCallback = function(response) {
            if (response.dataset && response.dataset.error) {
                ScreenService.showMessage(response.dataset.error, 'ERROR');
            }
    
            if (response.dataset && response.dataset.GROUPREL[0] == false) {
                if (wdgLogotipo.dataSource.data[0].DSDIRLOGOMARCA) {
                    ScreenService.confirmMessage(Messages.CONFIRMAR_DELETE, 'QUESTION',
                        function() {
                            General.callRestEngine("product#/parcnegocio_logotipo/excluirLogo", "Row", wdgLogotipo.currentRow).then(
                                function() {
                                    wdgLogotipo.currentRow.DSDIRLOGOMARCA = null;
                                    wdgLogotipo.currentRow.DSDIRLOGOMARCA_OLD = null;
                                    ScreenService.showMessage("Logotipo excluído com sucesso!");
                                    this.controlarAcaoExcluirLogo(wdgLogotipo);
                                    this.setDefaultImage(wdgLogotipo);
                                }.bind(this)
                            ).catch(
                                function(error) {
                                    console.error(error);
                                    ScreenService.showMessage(error.data.error);
                                }
                            );
                        }.bind(this)
                    );
                } else {
                    ScreenService.showMessage('Nenhuma logo encontrada para este parceiro!');
                }
            } else {
                ScreenService.showMessage(
                    'Operação Bloqueada! O parceiro de negócio está relacionado ao parceiro:<br>' +
                    response.dataset.GROUPREL[0]['NRINSCRICAOPARC'] +
                    ' - ' +
                    response.dataset.GROUPREL[0]['NMPRINCIPALPARC'] +
                    '!'
                );
            }
        }.bind(this);
    
        if (tipoParc == 'PROSPECT' || tipoParc == 'CLIENTE') {
            this.checkIfCanEditOrEvolParc(nrParcNegocio).then(
                function(canEdit) {
                    if (canEdit) {
                        General.callRestEngine("product#/verifyGroupRel", "DataSet", [currentRow]).then(
                            verifyGroupRelCallback
                        );
                    } else {
                        this.showMessageCantEditParc();
                    }
                }.bind(this)
            );
        } else {
            General.callRestEngine("product#/verifyGroupRel", "DataSet", [currentRow]).then(
                verifyGroupRelCallback
            );
        }
    };
    
    this.getWidgetsMenosLogo = function() {
        var wdgLogotipo = templateManager.container.getWidget("WDGT_LOGO_PARCEIRO");
        
        var widgets = wdgLogotipo.getParent().widgets;
		widgets = widgets.filter(function(wid) {
		    return wid.name !== wdgLogotipo.name;
		});
		
		return widgets.concat(wdgLogotipo.getParent());
    };
    
    this.showMessageCantEditParc = function() {
        var message = 'Ação bloqueada!<br>' +
            'Somente o operador responsável pelo prospect e seus assistentes podem ' +
            'editá-lo/evoluí-lo.';
    
        ScreenService.showMessage(message);
    };
    
    this.checkIfCanEditOrEvolParc = function(nrParcNegocio) {
        var restRow = {
            NRPARCNEGOCIO: nrParcNegocio,
            CRM_NRORGTRAB: getLocalVar('CRM_NRORGTRABALHO')
        };
    
        var params = {
            requestType: 'Row',
            row: restRow
        };
    
        return restEngine.post('/checkIfCanEditOrEvolParc', params).then(function(response) {
            return response.dataset.DADOS.CAN_EDIT;
        }).catch(function() {
            return false;
        });
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ABALogotipoParceiroController', ABALogotipoParceiroController);
});